@@ -1,7 +1,7 @@
 # Creates graphical rectangle objects out of coordinates
 createRec <-
 function(datlist, filled, label, bg.labels=220, labellb, lwd, inflate.labels, force.print.labels, 
-         cex_index, border.col, fontcolor.labels, fontface.labels, fontfamily.labels, align.labels, xmod.labels, ymod.labels) {
+         cex_index, border.col, fontcolor.labels, fontface.labels, fontfamily.labels, align.labels, xmod.labels, ymod.labels, eval.labels) {
 #browser()
 	if (nrow(datlist)==0) {
 		return(list(recs=NA, txt=NA, txtbg=NA))
@@ -36,7 +36,7 @@ function(datlist, filled, label, bg.labels=220, labellb, lwd, inflate.labels, fo
 		noText <- recs$name == ""
 		recs$name[noText] <- " "
 		txt <- str2rect(recs, fontcol=tCol, fill=txtfill, fontface=fontface.labels, fontfamily=fontfamily.labels, inflate.labels=inflate.labels, 
-                        cex_index=cex_index, align.labels=align.labels, xmod.labels=xmod.labels, ymod.labels=ymod.labels)
+                        cex_index=cex_index, align.labels=align.labels, xmod.labels=xmod.labels, ymod.labels=ymod.labels, eval.labels=eval.labels)
 		
         
         txt$txt$gp$col[noText] <- NA
